@@ -2,9 +2,11 @@ extern crate base64;
 extern crate crossbeam_utils;
 extern crate env_logger;
 extern crate flate2;
-extern crate hyperx;
 extern crate jsonwebtoken as jwt;
+
+#[cfg(not(target_os = "freebsd"))]
 extern crate libmount;
+
 #[macro_use]
 extern crate log;
 extern crate nix;
@@ -32,6 +34,7 @@ use sccache::dist::{
     UpdateJobStateResult,
 };
 use sccache::util::daemonize;
+use sccache::util::BASE64_URL_SAFE_ENGINE;
 use std::collections::{btree_map, BTreeMap, HashMap, HashSet};
 use std::env;
 use std::io;
@@ -40,7 +43,9 @@ use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Mutex, MutexGuard};
 use std::time::{Duration, Instant};
 
+#[cfg_attr(target_os = "freebsd", path = "build_freebsd.rs")]
 mod build;
+
 mod cmdline;
 mod token_check;
 
@@ -48,8 +53,11 @@ use cmdline::{AuthSubcommand, Command};
 
 pub const INSECURE_DIST_SERVER_TOKEN: &str = "dangerously_insecure_server";
 
-// Only supported on x86_64 Linux machines
-#[cfg(all(target_os = "linux", target_arch = "x86_64"))]
+// Only supported on x86_64 Linux machines and on FreeBSD
+#[cfg(any(
+    all(target_os = "linux", target_arch = "x86_64"),
+    target_os = "freebsd"
+))]
 fn main() {
     init_logging();
 
@@ -136,7 +144,7 @@ fn run(command: Command) -> Result<i32> {
             let mut bytes = vec![0; num_bytes];
             OsRng.fill_bytes(&mut bytes);
             // As long as it can be copied, it doesn't matter if this is base64 or hex etc
-            println!("{}", base64::encode_config(&bytes, base64::URL_SAFE_NO_PAD));
+            println!("{}", base64::encode_engine(&bytes, &BASE64_URL_SAFE_ENGINE));
             Ok(0)
         }
         Command::Auth(AuthSubcommand::JwtHS256ServerToken {
@@ -144,7 +152,7 @@ fn run(command: Command) -> Result<i32> {
             server_id,
         }) => {
             let header = jwt::Header::new(jwt::Algorithm::HS256);
-            let secret_key = base64::decode_config(&secret_key, base64::URL_SAFE_NO_PAD)?;
+            let secret_key = base64::decode_engine(&secret_key, &BASE64_URL_SAFE_ENGINE)?;
             let token = create_jwt_server_token(server_id, &header, &secret_key)
                 .context("Failed to create server token")?;
             println!("{}", token);
@@ -189,7 +197,7 @@ fn run(command: Command) -> Result<i32> {
                     Box::new(move |server_token| check_server_token(server_token, &token))
                 }
                 scheduler_config::ServerAuth::JwtHS256 { secret_key } => {
-                    let secret_key = base64::decode_config(&secret_key, base64::URL_SAFE_NO_PAD)
+                    let secret_key = base64::decode_engine(&secret_key, &BASE64_URL_SAFE_ENGINE)
                         .context("Secret key base64 invalid")?;
                     if secret_key.len() != 256 / 8 {
                         bail!("Size of secret key incorrect")
@@ -227,9 +235,11 @@ fn run(command: Command) -> Result<i32> {
             toolchain_cache_size,
         }) => {
             let builder: Box<dyn dist::BuilderIncoming> = match builder {
+                #[cfg(not(target_os = "freebsd"))]
                 server_config::BuilderType::Docker => {
                     Box::new(build::DockerBuilder::new().context("Docker builder failed to start")?)
                 }
+                #[cfg(not(target_os = "freebsd"))]
                 server_config::BuilderType::Overlay {
                     bwrap_path,
                     build_dir,
@@ -237,6 +247,23 @@ fn run(command: Command) -> Result<i32> {
                     build::OverlayBuilder::new(bwrap_path, build_dir)
                         .context("Overlay builder failed to start")?,
                 ),
+                #[cfg(target_os = "freebsd")]
+                server_config::BuilderType::Pot {
+                    pot_fs_root,
+                    clone_from,
+                    pot_cmd,
+                    pot_clone_args,
+                } => Box::new(
+                    build::PotBuilder::new(pot_fs_root, clone_from, pot_cmd, pot_clone_args)
+                        .context("Pot builder failed to start")?,
+                ),
+                _ => bail!(
+                    "Builder type `{}` not supported on this platform",
+                    format!("{:?}", builder)
+                        .split_whitespace()
+                        .next()
+                        .unwrap_or("")
+                ),
             };
 
             let server_id = ServerId::new(public_addr);
