@@ -24,11 +24,11 @@ use crate::dist;
 use crate::dist::pkg;
 use crate::mock_command::CommandCreatorSync;
 use crate::util::{hash_all, Digest, HashToDigest};
+use fs_err as fs;
 use std::borrow::Cow;
 use std::collections::{HashMap, HashSet};
 use std::ffi::{OsStr, OsString};
 use std::fmt;
-use std::fs;
 use std::hash::Hash;
 #[cfg(feature = "dist-client")]
 use std::io;
@@ -98,6 +98,10 @@ pub struct ParsedArguments {
     pub preprocessor_args: Vec<OsString>,
     /// Commandline arguments for the preprocessor or the compiler.
     pub common_args: Vec<OsString>,
+    /// Commandline arguments for the compiler that specify the architecture given
+    pub arch_args: Vec<OsString>,
+    /// Commandline arguments for the preprocessor or the compiler that don't affect the computed hash.
+    pub unhashed_args: Vec<OsString>,
     /// Extra files that need to have their contents hashed.
     pub extra_hash_files: Vec<PathBuf>,
     /// Whether or not the `-showIncludes` argument is passed on MSVC
@@ -382,17 +386,23 @@ where
             preprocessor_result.stdout.len()
         );
 
+        // Create an argument vector containing both common and arch args, to
+        // use in creating a hash key
+        let mut common_and_arch_args = parsed_args.common_args.clone();
+        common_and_arch_args.extend(parsed_args.arch_args.to_vec());
+
         let key = {
             hash_key(
                 &executable_digest,
                 parsed_args.language,
-                &parsed_args.common_args,
+                &common_and_arch_args,
                 &extra_hashes,
                 &env_vars,
                 &preprocessor_result.stdout,
                 compiler.plusplus(),
             )
         };
+
         // A compiler binary may be a symlink to another and so has the same digest, but that means
         // the toolchain will not contain the correct path to invoke the compiler! Add the compiler
         // executable path to try and prevent this
@@ -673,7 +683,7 @@ impl pkg::ToolchainPackager for CToolchainPackager {
 }
 
 /// The cache is versioned by the inputs to `hash_key`.
-pub const CACHE_VERSION: &[u8] = b"10";
+pub const CACHE_VERSION: &[u8] = b"11";
 
 lazy_static! {
     /// Environment variables that are factored into the cache key.
