@@ -159,6 +159,7 @@ impl CCompilerImpl for Clang {
 }
 
 counted_array!(pub static ARGS: [ArgInfo<gcc::ArgData>; _] = [
+    take_arg!("--dependent-lib", OsString, Concatenated('='), PassThrough),
     take_arg!("--serialize-diagnostics", OsString, Separated, PassThrough),
     take_arg!("--target", OsString, Separated, PassThrough),
     // Note: for clang we must override the dep options from gcc.rs with `CanBeSeparated`.
@@ -183,7 +184,9 @@ counted_array!(pub static ARGS: [ArgInfo<gcc::ArgData>; _] = [
     // Note: this overrides the -fprofile-use option in gcc.rs.
     take_arg!("-fprofile-use", PathBuf, Concatenated('='), ClangProfileUse),
     take_arg!("-fsanitize-blacklist", PathBuf, Concatenated('='), ExtraHashFile),
+    flag!("-fuse-ctor-homing", PassThroughFlag),
     take_arg!("-gcc-toolchain", OsString, Separated, PassThrough),
+    flag!("-gcodeview", PassThroughFlag),
     take_arg!("-include-pch", PathBuf, CanBeSeparated, PreprocessorArgumentPath),
     take_arg!("-load", PathBuf, Separated, ExtraHashFile),
     take_arg!("-mllvm", OsString, Separated, PassThrough),
@@ -293,10 +296,33 @@ mod test {
             )
         );
         assert_eq!(ovec!["-Iinclude", "-include", "file"], a.preprocessor_args);
-        assert_eq!(
-            ovec!["-arch", "xyz", "-fabc", "/winsysroot", "../some/dir"],
-            a.common_args
+        assert_eq!(ovec!["-fabc", "/winsysroot", "../some/dir"], a.common_args);
+        assert_eq!(ovec!["-arch", "xyz"], a.arch_args);
+    }
+
+    #[test]
+    fn test_dependent_lib() {
+        let a = parses!(
+            "-c",
+            "foo.c",
+            "-o",
+            "foo.o",
+            "-Xclang",
+            "--dependent-lib=msvcrt"
         );
+        assert_eq!(Some("foo.c"), a.input.to_str());
+        assert_eq!(Language::C, a.language);
+        assert_map_contains!(
+            a.outputs,
+            (
+                "obj",
+                ArtifactDescriptor {
+                    path: PathBuf::from("foo.o"),
+                    optional: false
+                }
+            )
+        );
+        assert_eq!(ovec!["-Xclang", "--dependent-lib=msvcrt"], a.common_args);
     }
 
     #[test]
@@ -313,6 +339,11 @@ mod test {
         parses!("-c", "foo.c", "-gcc-toolchain", "somewhere", "-o", "foo.o");
     }
 
+    #[test]
+    fn test_gcodeview() {
+        parses!("-c", "foo.c", "-o", "foo.o", "-Xclang", "-gcodeview");
+    }
+
     #[test]
     fn test_parse_clang_short_dependency_arguments_can_be_separated() {
         let args = vec!["-MF", "-MT", "-MQ"];
@@ -354,7 +385,10 @@ mod test {
             ])
         );
         assert_eq!(
-            CompilerArguments::CannotCache("Can't handle UnknownFlag arguments with -Xclang", None),
+            CompilerArguments::CannotCache(
+                "Can't handle UnknownFlag arguments with -Xclang",
+                Some("-broken".to_string())
+            ),
             parse_arguments_(stringvec![
                 "-c", "foo.c", "-o", "foo.o", "-Xclang", "-broken"
             ])
@@ -517,6 +551,12 @@ mod test {
         assert_eq!(ovec!["-Xclang", "-no-opaque-pointers"], a.preprocessor_args);
     }
 
+    #[test]
+    fn test_parse_xclang_use_ctor_homing() {
+        let a = parses!("-c", "foo.c", "-o", "foo.o", "-Xclang", "-fuse-ctor-homing");
+        assert_eq!(ovec!["-Xclang", "-fuse-ctor-homing"], a.common_args);
+    }
+
     #[test]
     fn test_parse_fplugin() {
         let a = parses!("-c", "foo.c", "-o", "foo.o", "-fplugin", "plugin.so");
