@@ -49,11 +49,20 @@ use tempfile::TempDir;
 use crate::errors::*;
 
 /// Can dylibs (shared libraries or proc macros) be distributed on this platform?
-#[cfg(all(feature = "dist-client", target_os = "linux", target_arch = "x86_64"))]
+#[cfg(all(
+    feature = "dist-client",
+    any(
+        all(target_os = "linux", target_arch = "x86_64"),
+        target_os = "freebsd"
+    )
+))]
 pub const CAN_DIST_DYLIBS: bool = true;
 #[cfg(all(
     feature = "dist-client",
-    not(all(target_os = "linux", target_arch = "x86_64"))
+    not(any(
+        all(target_os = "linux", target_arch = "x86_64"),
+        target_os = "freebsd"
+    ))
 ))]
 pub const CAN_DIST_DYLIBS: bool = false;
 
@@ -1048,7 +1057,7 @@ g++
 gcc
 #elif defined(__DCC__)
 diab
-#elif definded(__CTC__)
+#elif defined(__CTC__)
 tasking_vx
 #else
 unknown
