@@ -113,6 +113,7 @@ ArgData! { pub
     NoDiagnosticsColorFlag,
     // Should only be necessary for -Xclang flags - unknown flags not hidden behind
     // that are assumed to not affect compilation
+    PassThroughFlag,
     PassThrough(OsString),
     PassThroughPath(PathBuf),
     PreprocessorArgumentFlag,
@@ -142,6 +143,8 @@ ArgData! { pub
 
 use self::ArgData::*;
 
+const ARCH_FLAG: &str = "-arch";
+
 // Mostly taken from https://github.com/ccache/ccache/blob/master/src/compopt.c#L32-L84
 counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     flag!("-", TooHardFlag),
@@ -174,7 +177,7 @@ counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     take_arg!("-Xassembler", OsString, Separated, PassThrough),
     take_arg!("-Xlinker", OsString, Separated, PassThrough),
     take_arg!("-Xpreprocessor", OsString, Separated, PreprocessorArgument),
-    take_arg!("-arch", OsString, Separated, Arch),
+    take_arg!(ARCH_FLAG, OsString, Separated, Arch),
     take_arg!("-aux-info", OsString, Separated, PassThrough),
     take_arg!("-b", OsString, Separated, PassThrough),
     flag!("-c", DoCompilation),
@@ -244,6 +247,7 @@ where
     let mut dep_target = None;
     let mut dep_flag = OsString::from("-MT");
     let mut common_args = vec![];
+    let mut arch_args = vec![];
     let mut preprocessor_args = vec![];
     let mut dependency_args = vec![];
     let mut extra_hash_files = vec![];
@@ -266,7 +270,6 @@ where
     let mut outputs_gcno = false;
     let mut xclangs: Vec<OsString> = vec![];
     let mut color_mode = ColorMode::Auto;
-    let mut seen_arch = None;
 
     // Custom iterator to expand `@` arguments which stand for reading a file
     // and interpreting it as a list of more arguments.
@@ -340,6 +343,7 @@ where
                 need_explicit_dep_argument_path = DepArgumentRequirePath::Provided
             }
             Some(ExtraHashFile(_))
+            | Some(PassThroughFlag)
             | Some(PreprocessorArgumentFlag)
             | Some(PreprocessorArgument(_))
             | Some(PreprocessorArgumentPath(_))
@@ -355,13 +359,7 @@ where
                     _ => cannot_cache!("-x"),
                 };
             }
-            Some(Arch(arch)) => {
-                match seen_arch {
-                    Some(s) if &s != arch => cannot_cache!("multiple different -arch"),
-                    _ => {}
-                };
-                seen_arch = Some(arch.clone());
-            }
+            Some(Arch(_)) => {}
             Some(XClang(s)) => xclangs.push(s.clone()),
             None => match arg {
                 Argument::Raw(ref val) => {
@@ -386,9 +384,10 @@ where
             | Some(DiagnosticsColor(_))
             | Some(DiagnosticsColorFlag)
             | Some(NoDiagnosticsColorFlag)
-            | Some(Arch(_))
+            | Some(PassThroughFlag)
             | Some(PassThrough(_))
             | Some(PassThroughPath(_)) => &mut common_args,
+            Some(Arch(_)) => &mut arch_args,
             Some(ExtraHashFile(path)) => {
                 extra_hash_files.push(cwd.join(path));
                 &mut common_args
@@ -440,8 +439,11 @@ where
             None => match arg {
                 Argument::Raw(_) if follows_plugin_arg => &mut common_args,
                 Argument::Raw(_) => cannot_cache!("Can't handle Raw arguments with -Xclang"),
-                Argument::UnknownFlag(_) => {
-                    cannot_cache!("Can't handle UnknownFlag arguments with -Xclang")
+                Argument::UnknownFlag(flag) => {
+                    cannot_cache!(
+                        "Can't handle UnknownFlag arguments with -Xclang",
+                        flag.to_str().unwrap_or("").to_string()
+                    )
                 }
                 _ => unreachable!(),
             },
@@ -450,6 +452,7 @@ where
             | Some(NoDiagnosticsColorFlag)
             | Some(Arch(_))
             | Some(PassThrough(_))
+            | Some(PassThroughFlag)
             | Some(PassThroughPath(_)) => &mut common_args,
             Some(ExtraHashFile(path)) => {
                 extra_hash_files.push(cwd.join(path));
@@ -567,6 +570,7 @@ where
         dependency_args,
         preprocessor_args,
         common_args,
+        arch_args,
         extra_hash_files,
         msvc_show_includes: false,
         profile_generate,
@@ -620,13 +624,30 @@ fn preprocess_cmd<T>(
             }
         }
     }
+
+    // Explicitly rewrite the -arch args to be preprocessor defines of the form
+    // __arch__ so that they affect the preprocessor output but don't cause
+    // clang to error.
+    debug!("arch args before rewrite: {:?}", parsed_args.arch_args);
+    let rewritten_arch_args = parsed_args
+        .arch_args
+        .iter()
+        .filter(|&arg| arg.ne(ARCH_FLAG))
+        .filter_map(|arg| {
+            arg.to_str()
+                .map(|arg_string| format!("-D__{}__=1", arg_string).into())
+        })
+        .collect::<Vec<OsString>>();
+
     cmd.arg(&parsed_args.input)
         .args(&parsed_args.preprocessor_args)
         .args(&parsed_args.dependency_args)
         .args(&parsed_args.common_args)
+        .args(&rewritten_arch_args)
         .env_clear()
         .envs(env_vars.iter().map(|&(ref k, ref v)| (k, v)))
         .current_dir(cwd);
+    debug!("cmd after -arch rewrite: {:?}", cmd);
 }
 
 #[allow(clippy::too_many_arguments)]
@@ -705,6 +726,7 @@ pub fn generate_compile_commands(
     ];
     arguments.extend(parsed_args.preprocessor_args.clone());
     arguments.extend(parsed_args.common_args.clone());
+    arguments.extend(parsed_args.arch_args.clone());
     let command = CompileCommand {
         executable: executable.to_owned(),
         arguments,
@@ -1391,6 +1413,40 @@ mod test {
         assert!(args.common_args.contains(&"-fdiagnostics-color".into()));
     }
 
+    #[test]
+    fn test_preprocess_cmd_rewrites_archs() {
+        let args = stringvec!["-arch", "arm64", "-arch", "i386", "-c", "foo.cc"];
+        let parsed_args = match parse_arguments_(args, false) {
+            CompilerArguments::Ok(args) => args,
+            o => panic!("Got unexpected parse result: {:?}", o),
+        };
+        let mut cmd = MockCommand {
+            child: None,
+            args: vec![],
+        };
+        preprocess_cmd(
+            &mut cmd,
+            &parsed_args,
+            Path::new(""),
+            &[],
+            true,
+            CCompilerKind::Gcc,
+            true,
+            vec![],
+        );
+        // make sure the architectures were rewritten to prepocessor defines
+        let expected_args = ovec![
+            "-x",
+            "c++",
+            "-E",
+            "-fdirectives-only",
+            "foo.cc",
+            "-D__arm64__=1",
+            "-D__i386__=1"
+        ];
+        assert_eq!(cmd.args, expected_args);
+    }
+
     #[test]
     fn pedantic_default() {
         let args = stringvec!["-pedantic", "-c", "foo.cc"];
@@ -1630,15 +1686,39 @@ mod test {
             o => panic!("Got unexpected parse result: {:?}", o),
         }
 
-        assert_eq!(
-            CompilerArguments::CannotCache("multiple different -arch", None),
-            parse_arguments_(
-                stringvec![
-                    "-fPIC", "-arch", "arm64", "-arch", "i386", "-o", "foo.o", "-c", "foo.cpp"
-                ],
-                false
+        let args =
+            stringvec!["-fPIC", "-arch", "arm64", "-arch", "i386", "-o", "foo.o", "-c", "foo.cpp"];
+        let ParsedArguments {
+            input,
+            language,
+            compilation_flag,
+            outputs,
+            preprocessor_args,
+            msvc_show_includes,
+            common_args,
+            arch_args,
+            ..
+        } = match parse_arguments_(args, false) {
+            CompilerArguments::Ok(args) => args,
+            o => panic!("Got unexpected parse result: {:?}", o),
+        };
+        assert_eq!(Some("foo.cpp"), input.to_str());
+        assert_eq!(Language::Cxx, language);
+        assert_eq!(Some("-c"), compilation_flag.to_str());
+        assert_map_contains!(
+            outputs,
+            (
+                "obj",
+                ArtifactDescriptor {
+                    path: "foo.o".into(),
+                    optional: false
+                }
             )
         );
+        assert!(preprocessor_args.is_empty());
+        assert_eq!(ovec!["-fPIC"], common_args);
+        assert_eq!(ovec!["-arch", "arm64", "-arch", "i386"], arch_args);
+        assert!(!msvc_show_includes);
     }
 
     #[test]
@@ -1706,6 +1786,7 @@ mod test {
             dependency_args: vec![],
             preprocessor_args: vec![],
             common_args: vec![],
+            arch_args: vec![],
             extra_hash_files: vec![],
             msvc_show_includes: false,
             profile_generate: false,
