@@ -494,6 +494,7 @@ msvc_args!(static ARGS: [ArgInfo<ArgData>; _] = [
     msvc_take_arg!("favor:", OsString, Concatenated, PassThroughWithSuffix),
     msvc_take_arg!("fp:", OsString, Concatenated, PassThroughWithSuffix),
     msvc_take_arg!("fsanitize-blacklist", PathBuf, Concatenated('='), ExtraHashFile),
+    msvc_flag!("fsanitize=address", PassThrough),
     msvc_flag!("fsyntax-only", SuppressCompilation),
     msvc_take_arg!("guard:cf", OsString, Concatenated, PassThroughWithSuffix),
     msvc_flag!("homeparams", PassThrough),
@@ -699,6 +700,7 @@ pub fn parse_arguments(
                 | Some(DiagnosticsColorFlag)
                 | Some(NoDiagnosticsColorFlag)
                 | Some(Arch(_))
+                | Some(PassThroughFlag)
                 | Some(PassThrough(_))
                 | Some(PassThroughPath(_))
                 | Some(PedanticFlag)
@@ -818,6 +820,7 @@ pub fn parse_arguments(
         dependency_args,
         preprocessor_args,
         common_args,
+        arch_args: vec![],
         extra_hash_files,
         msvc_show_includes: show_includes,
         profile_generate,
@@ -1622,6 +1625,7 @@ mod test {
             dependency_args: vec![],
             preprocessor_args: vec![],
             common_args: vec![],
+            arch_args: vec![],
             extra_hash_files: vec![],
             msvc_show_includes: false,
             profile_generate: false,
@@ -1681,6 +1685,7 @@ mod test {
             dependency_args: vec![],
             preprocessor_args: vec![],
             common_args: vec![],
+            arch_args: vec![],
             extra_hash_files: vec![],
             msvc_show_includes: false,
             profile_generate: false,
