@@ -146,6 +146,8 @@ impl HTTPUrl {
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct AzureCacheConfig {
+    pub connection_string: String,
+    pub container: String,
     pub key_prefix: String,
 }
 
@@ -181,10 +183,10 @@ pub enum GCSCacheRWMode {
 pub struct GCSCacheConfig {
     pub bucket: String,
     pub key_prefix: String,
-    pub cred_path: Option<PathBuf>,
-    pub oauth_url: Option<String>,
-    pub deprecated_url: Option<String>,
+    pub cred_path: Option<String>,
+    pub service_account: Option<String>,
     pub rw_mode: GCSCacheRWMode,
+    pub credential_url: Option<String>,
 }
 
 #[derive(Debug, PartialEq, Eq, Serialize, Deserialize)]
@@ -242,9 +244,9 @@ pub struct CacheConfigs {
 }
 
 impl CacheConfigs {
-    /// Return a vec of the available cache types in an arbitrary but
+    /// Return cache type in an arbitrary but
     /// consistent ordering
-    fn into_vec_and_fallback(self) -> (Vec<CacheType>, DiskCacheConfig) {
+    fn into_fallback(self) -> (Option<CacheType>, DiskCacheConfig) {
         let CacheConfigs {
             azure,
             disk,
@@ -255,18 +257,17 @@ impl CacheConfigs {
             s3,
         } = self;
 
-        let caches = s3
+        let cache_type = s3
             .map(CacheType::S3)
-            .into_iter()
-            .chain(redis.map(CacheType::Redis))
-            .chain(memcached.map(CacheType::Memcached))
-            .chain(gcs.map(CacheType::GCS))
-            .chain(gha.map(CacheType::GHA))
-            .chain(azure.map(CacheType::Azure))
-            .collect();
+            .or_else(|| redis.map(CacheType::Redis))
+            .or_else(|| memcached.map(CacheType::Memcached))
+            .or_else(|| gcs.map(CacheType::GCS))
+            .or_else(|| gha.map(CacheType::GHA))
+            .or_else(|| azure.map(CacheType::Azure));
+
         let fallback = disk.unwrap_or_default();
 
-        (caches, fallback)
+        (cache_type, fallback)
     }
 
     /// Override self with any existing fields from other
@@ -533,20 +534,19 @@ fn config_from_env() -> Result<EnvConfig> {
             .filter(|s| !s.is_empty())
             .unwrap_or_default()
             .to_owned();
-        let deprecated_url = env::var("SCCACHE_GCS_CREDENTIALS_URL").ok();
-        let oauth_url = env::var("SCCACHE_GCS_OAUTH_URL").ok();
-        let cred_path = env::var_os("SCCACHE_GCS_KEY_PATH").map(PathBuf::from);
 
-        if oauth_url.is_some() && cred_path.is_some() {
-            warn!("Both SCCACHE_GCS_OAUTH_URL and SCCACHE_GCS_KEY_PATH are set");
-            warn!("You should set only one of them!");
-            warn!("SCCACHE_GCS_KEY_PATH will take precedence");
-        }
-        if let Some(p) = &cred_path {
-            if !p.is_file() {
-                warn!("Could not find SCCACHE_GCS_KEY_PATH file '{:?}'", p);
-            }
+
+
+        if env::var("SCCACHE_GCS_OAUTH_URL").is_ok() {
+            warn!("SCCACHE_GCS_OAUTH_URL has been deprecated");
+            warn!("if you intend to use vm metadata for auth, please set correct service account intead");
         }
+
+        let credential_url = env::var("SCCACHE_GCS_CREDENTIALS_URL").ok();
+
+        let cred_path = env::var("SCCACHE_GCS_KEY_PATH").ok();
+        let service_account = env::var("SCCACHE_GCS_SERVICE_ACCOUNT").ok();
+
         let rw_mode = match env::var("SCCACHE_GCS_RW_MODE").as_ref().map(String::as_str) {
             Ok("READ_ONLY") => GCSCacheRWMode::ReadOnly,
             Ok("READ_WRITE") => GCSCacheRWMode::ReadWrite,
@@ -561,13 +561,14 @@ fn config_from_env() -> Result<EnvConfig> {
                 GCSCacheRWMode::ReadOnly
             }
         };
+
         GCSCacheConfig {
             bucket,
             key_prefix,
             cred_path,
-            oauth_url,
-            deprecated_url,
+            service_account,
             rw_mode,
+            credential_url,
         }
     });
 
@@ -591,7 +592,10 @@ fn config_from_env() -> Result<EnvConfig> {
     };
 
     // ======= Azure =======
-    let azure = env::var("SCCACHE_AZURE_CONNECTION_STRING").ok().map(|_| {
+    let azure = if let (Ok(connection_string), Ok(container)) = (
+        env::var("SCCACHE_AZURE_CONNECTION_STRING"),
+        env::var("SCCACHE_AZURE_BLOB_CONTAINER"),
+    ) {
         let key_prefix = env::var("SCCACHE_AZURE_KEY_PREFIX")
             .ok()
             .as_ref()
@@ -599,8 +603,14 @@ fn config_from_env() -> Result<EnvConfig> {
             .filter(|s| !s.is_empty())
             .unwrap_or_default()
             .to_owned();
-        AzureCacheConfig { key_prefix }
-    });
+        Some(AzureCacheConfig {
+            connection_string,
+            container,
+            key_prefix,
+        })
+    } else {
+        None
+    };
 
     // ======= Local =======
     let disk_dir = env::var_os("SCCACHE_DIR").map(PathBuf::from);
@@ -655,7 +665,7 @@ fn config_file(env_var: &str, leaf: &str) -> PathBuf {
 
 #[derive(Debug, Default, PartialEq, Eq)]
 pub struct Config {
-    pub caches: Vec<CacheType>,
+    pub cache: Option<CacheType>,
     pub fallback_cache: DiskCacheConfig,
     pub dist: DistConfig,
     pub server_startup_timeout: Option<std::time::Duration>,
@@ -689,9 +699,9 @@ impl Config {
         let EnvConfig { cache } = env_conf;
         conf_caches.merge(cache);
 
-        let (caches, fallback_cache) = conf_caches.into_vec_and_fallback();
+        let (caches, fallback_cache) = conf_caches.into_fallback();
         Self {
-            caches,
+            cache: caches,
             fallback_cache,
             dist,
             server_startup_timeout,
@@ -851,6 +861,30 @@ pub mod server {
         TEN_GIGS
     }
 
+    const DEFAULT_POT_CLONE_FROM: &str = "sccache-template";
+    const DEFAULT_POT_FS_ROOT: &str = "/opt/pot";
+    const DEFAULT_POT_CMD: &str = "pot";
+    const DEFAULT_POT_CLONE_ARGS: &[&str] = &["-i", "lo0|127.0.0.2"];
+
+    fn default_pot_clone_from() -> String {
+        DEFAULT_POT_CLONE_FROM.to_string()
+    }
+
+    fn default_pot_fs_root() -> PathBuf {
+        DEFAULT_POT_FS_ROOT.into()
+    }
+
+    fn default_pot_cmd() -> PathBuf {
+        DEFAULT_POT_CMD.into()
+    }
+
+    fn default_pot_clone_args() -> Vec<String> {
+        DEFAULT_POT_CLONE_ARGS
+            .iter()
+            .map(|s| s.to_string())
+            .collect()
+    }
+
     #[derive(Debug, Serialize, Deserialize)]
     #[serde(tag = "type")]
     #[serde(deny_unknown_fields)]
@@ -862,6 +896,17 @@ pub mod server {
             build_dir: PathBuf,
             bwrap_path: PathBuf,
         },
+        #[serde(rename = "pot")]
+        Pot {
+            #[serde(default = "default_pot_fs_root")]
+            pot_fs_root: PathBuf,
+            #[serde(default = "default_pot_clone_from")]
+            clone_from: String,
+            #[serde(default = "default_pot_cmd")]
+            pot_cmd: PathBuf,
+            #[serde(default = "default_pot_clone_args")]
+            pot_clone_args: Vec<String>,
+        },
     }
 
     #[derive(Debug, Serialize, Deserialize)]
@@ -908,7 +953,9 @@ fn config_overrides() {
     let env_conf = EnvConfig {
         cache: CacheConfigs {
             azure: Some(AzureCacheConfig {
-                key_prefix: "".to_owned(),
+                connection_string: String::new(),
+                container: String::new(),
+                key_prefix: String::new(),
             }),
             disk: Some(DiskCacheConfig {
                 dir: "/env-cache".into(),
@@ -942,17 +989,9 @@ fn config_overrides() {
     assert_eq!(
         Config::from_env_and_file_configs(env_conf, file_conf),
         Config {
-            caches: vec![
-                CacheType::Redis(RedisCacheConfig {
-                    url: "myotherredisurl".to_owned()
-                }),
-                CacheType::Memcached(MemcachedCacheConfig {
-                    url: "memurl".to_owned()
-                }),
-                CacheType::Azure(AzureCacheConfig {
-                    key_prefix: "".to_owned()
-                }),
-            ],
+            cache: Some(CacheType::Redis(RedisCacheConfig {
+                url: "myotherredisurl".to_owned()
+            }),),
             fallback_cache: DiskCacheConfig {
                 dir: "/env-cache".into(),
                 size: 5,
@@ -984,53 +1023,29 @@ fn test_s3_no_credentials() {
 }
 
 #[test]
-fn test_gcs_credentials_url() {
+fn test_gcs_service_account() {
     env::set_var("SCCACHE_GCS_BUCKET", "my-bucket");
-    env::set_var("SCCACHE_GCS_CREDENTIALS_URL", "http://localhost/");
+    env::set_var("SCCACHE_GCS_SERVICE_ACCOUNT", "my@example.com");
     env::set_var("SCCACHE_GCS_RW_MODE", "READ_WRITE");
 
     let env_cfg = config_from_env().unwrap();
     match env_cfg.cache.gcs {
         Some(GCSCacheConfig {
             ref bucket,
-            ref deprecated_url,
+            service_account,
             rw_mode,
             ..
         }) => {
             assert_eq!(bucket, "my-bucket");
-            match deprecated_url {
-                Some(ref url) => assert_eq!(url, "http://localhost/"),
-                None => panic!("URL can't be none"),
-            };
+            assert_eq!(service_account, Some("my@example.com".to_string()));
             assert_eq!(rw_mode, GCSCacheRWMode::ReadWrite);
         }
         None => unreachable!(),
     };
-}
 
-#[test]
-fn test_gcs_oauth_url() {
-    env::set_var("SCCACHE_GCS_BUCKET", "my-bucket");
-    env::set_var("SCCACHE_GCS_OAUTH_URL", "http://localhost/");
-    env::set_var("SCCACHE_GCS_RW_MODE", "READ_WRITE");
-
-    let env_cfg = config_from_env().unwrap();
-    match env_cfg.cache.gcs {
-        Some(GCSCacheConfig {
-            ref bucket,
-            ref oauth_url,
-            rw_mode,
-            ..
-        }) => {
-            assert_eq!(bucket, "my-bucket");
-            match oauth_url {
-                Some(ref url) => assert_eq!(url, "http://localhost/"),
-                None => panic!("URL can't be none"),
-            };
-            assert_eq!(rw_mode, GCSCacheRWMode::ReadWrite);
-        }
-        None => unreachable!(),
-    };
+    env::remove_var("SCCACHE_GCS_BUCKET");
+    env::remove_var("SCCACHE_GCS_SERVICE_ACCOUNT");
+    env::remove_var("SCCACHE_GCS_RW_MODE");
 }
 
 #[test]
@@ -1060,13 +1075,12 @@ dir = "/tmp/.cache/sccache"
 size = 7516192768 # 7 GiBytes
 
 [cache.gcs]
-# optional url
-deprecated_url = "..."
 rw_mode = "READ_ONLY"
 # rw_mode = "READ_WRITE"
 cred_path = "/psst/secret/cred"
 bucket = "bucket"
 key_prefix = "prefix"
+service_account = "example_service_account"
 
 [cache.gha]
 url = "http://localhost"
@@ -1100,12 +1114,12 @@ no_credentials = true
                     size: 7 * 1024 * 1024 * 1024,
                 }),
                 gcs: Some(GCSCacheConfig {
-                    oauth_url: None,
-                    deprecated_url: Some("...".to_owned()),
                     bucket: "bucket".to_owned(),
-                    cred_path: Some(PathBuf::from("/psst/secret/cred")),
+                    cred_path: Some("/psst/secret/cred".to_string()),
+                    service_account: Some("example_service_account".to_string()),
                     rw_mode: GCSCacheRWMode::ReadOnly,
                     key_prefix: "prefix".into(),
+                    credential_url: None,
                 }),
                 gha: Some(GHACacheConfig {
                     url: "http://localhost".to_owned(),
