@@ -29,6 +29,13 @@ use std::time::Duration;
 
 use crate::errors::*;
 
+/// The url safe engine for base64.
+pub const BASE64_URL_SAFE_ENGINE: base64::engine::fast_portable::FastPortable =
+    base64::engine::fast_portable::FastPortable::from(
+        &base64::alphabet::URL_SAFE,
+        base64::engine::fast_portable::NO_PAD,
+    );
+
 #[derive(Clone)]
 pub struct Digest {
     inner: blake3_Hasher,
@@ -411,115 +418,6 @@ pub fn ref_env(env: &[(OsString, OsString)]) -> impl Iterator<Item = (&OsString,
     env.iter().map(|&(ref k, ref v)| (k, v))
 }
 
-#[cfg(feature = "hyperx")]
-pub use self::http_extension::{HeadersExt, RequestExt};
-
-#[cfg(feature = "hyperx")]
-mod http_extension {
-    use reqwest::header::{HeaderMap, HeaderValue};
-    use std::fmt;
-
-    pub trait HeadersExt {
-        fn set<H>(&mut self, header: H)
-        where
-            H: hyperx::header::Header + fmt::Display;
-
-        fn get_hyperx<H>(&self) -> Option<H>
-        where
-            H: hyperx::header::Header;
-    }
-
-    impl HeadersExt for HeaderMap {
-        fn set<H>(&mut self, header: H)
-        where
-            H: hyperx::header::Header + fmt::Display,
-        {
-            self.insert(
-                H::header_name(),
-                HeaderValue::from_maybe_shared(header.to_string()).unwrap(),
-            );
-        }
-
-        fn get_hyperx<H>(&self) -> Option<H>
-        where
-            H: hyperx::header::Header,
-        {
-            http::HeaderMap::get(self, H::header_name())
-                .and_then(|header| H::parse_header(&header).ok())
-        }
-    }
-
-    pub trait RequestExt {
-        fn set_header<H>(self, header: H) -> Self
-        where
-            H: hyperx::header::Header + fmt::Display;
-    }
-
-    impl RequestExt for http::request::Builder {
-        fn set_header<H>(self, header: H) -> Self
-        where
-            H: hyperx::header::Header + fmt::Display,
-        {
-            self.header(
-                H::header_name(),
-                HeaderValue::from_maybe_shared(header.to_string()).unwrap(),
-            )
-        }
-    }
-
-    impl RequestExt for http::response::Builder {
-        fn set_header<H>(self, header: H) -> Self
-        where
-            H: hyperx::header::Header + fmt::Display,
-        {
-            self.header(
-                H::header_name(),
-                HeaderValue::from_maybe_shared(header.to_string()).unwrap(),
-            )
-        }
-    }
-
-    #[cfg(feature = "reqwest")]
-    impl RequestExt for ::reqwest::RequestBuilder {
-        fn set_header<H>(self, header: H) -> Self
-        where
-            H: hyperx::header::Header + fmt::Display,
-        {
-            self.header(
-                H::header_name(),
-                HeaderValue::from_maybe_shared(header.to_string()).unwrap(),
-            )
-        }
-    }
-
-    #[cfg(feature = "reqwest")]
-    impl RequestExt for ::reqwest::blocking::RequestBuilder {
-        fn set_header<H>(self, header: H) -> Self
-        where
-            H: hyperx::header::Header + fmt::Display,
-        {
-            self.header(
-                H::header_name(),
-                HeaderValue::from_maybe_shared(header.to_string()).unwrap(),
-            )
-        }
-    }
-}
-
-pub trait DateTimeExt {
-    fn to_rfc7231(&self) -> String;
-}
-
-#[cfg(any(feature = "azure", feature = "gcs"))]
-impl<Tz: chrono::TimeZone> DateTimeExt for chrono::DateTime<Tz>
-where
-    Tz::Offset: core::fmt::Display,
-{
-    fn to_rfc7231(&self) -> String {
-        self.naive_utc().format("%a, %d %b %Y %T GMT").to_string()
-    }
-}
-
 /// Pipe `cmd`'s stdio to `/dev/null`, unless a specific env var is set.
 #[cfg(not(windows))]
 pub fn daemonize() -> Result<()> {
@@ -639,20 +537,4 @@ mod tests {
         assert_eq!(a.split_prefix("foo2"), None);
         assert_eq!(a.split_prefix("b"), None);
     }
-
-    #[cfg(feature = "azure")]
-    #[test]
-    fn rfc7231_format() {
-        use crate::util::DateTimeExt;
-        use chrono::{DateTime, Utc};
-
-        let zero_datetime = match chrono::NaiveDateTime::from_timestamp_opt(0, 0) {
-            Some(v) => v,
-            _ => panic!("zero date time from_timetamp_opt"),
-        };
-
-        let time = DateTime::<Utc>::from_utc(zero_datetime, Utc);
-
-        assert_eq!(time.to_rfc7231(), "Thu, 01 Jan 1970 00:00:00 GMT");
-    }
 }
